/*
    SPDX-FileCopyrightText: 2006 Volker Krause <vkrause@kde.org>
                  2007 Till Adam <adam@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "messageparts.h"

const char Akonadi::MessagePart::Envelope[] = "ENVELOPE";
const char Akonadi::MessagePart::Body[] = "RFC822";
const char Akonadi::MessagePart::Header[] = "HEAD";
