/*
    SPDX-FileCopyrightText: 2009 Igor Trindade Oliveira <igor_trindade@yahoo.com.br>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef TESTMAILDIR_H
#define TESTMAILDIR_H

#include "test.h"

class TestMailDir : public Test
{
public:
    TestMailDir(const QString &dir);
};
#endif
