/*
    SPDX-FileCopyrightText: 2009 Igor Trindade Oliveira <igor_trindade@yahoo.com.br>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef TESTVCARD_H
#define TESTVCARD_H

#include "test.h"

class TestVCard : public Test
{
public:
    TestVCard(const QString &dir);
};
#endif
